@@ -57,25 +57,27 @@ class KMM(BaseAdaptEstimator):
     target data to the training set.
 
     Parameters
-    ----------        
-    B: float (default=1000)
-        Bounding weights parameter.
-        
-    eps: float, optional (default=None)
-        Constraint parameter.
-        If ``None`` eps is set to
-        ``(np.sqrt(len(Xs)) - 1)/np.sqrt(len(Xs))``
-        with ``Xs`` the source input dataset.
-        
+    ----------
     kernel : str (default="rbf")
         Kernel metric.
         Possible values: [‘additive_chi2’, ‘chi2’,
         ‘linear’, ‘poly’, ‘polynomial’, ‘rbf’,
         ‘laplacian’, ‘sigmoid’, ‘cosine’]
+    
+    B: float (default=1000)
+        Bounding weights parameter.
+        
+    eps: float, optional (default=None)
+        Constraint parameter.
+        If ``None``,  ``eps`` is set to::
+        
+            eps = (np.sqrt(len(Xs)) - 1)/np.sqrt(len(Xs))
+
+        with ``Xs`` the source input dataset.
         
     max_size : int (default=1000)
         Batch computation to speed up the fitting.
-        If len(Xs) > ``max_size``, KMM is applied
+        If ``len(Xs) > max_size``, KMM is applied
         successively on seperated batch
         of size lower than ``max_size``.
         
@@ -85,7 +87,43 @@ class KMM(BaseAdaptEstimator):
         
     max_iter: int (default=100)
         Maximal iteration of the optimization.
+        
+    Yields
+    ------
+    gamma : float
+        Kernel parameter ``gamma``.
+        
+        - For kernel = chi2::
+        
+            k(x, y) = exp(-gamma Sum [(x - y)^2 / (x + y)])
 
+        - For kernel = poly or polynomial::
+        
+            K(X, Y) = (gamma <X, Y> + coef0)^degree
+            
+        - For kernel = rbf::
+        
+            K(x, y) = exp(-gamma ||x-y||^2)
+        
+        - For kernel = laplacian::
+        
+            K(x, y) = exp(-gamma ||x-y||_1)
+        
+        - For kernel = sigmoid::
+        
+            K(X, Y) = tanh(gamma <X, Y> + coef0)
+        
+    coef0 : floaf
+        Kernel parameter ``coef0``.
+        Used for ploynomial and sigmoid kernels.
+        See ``gamma`` parameter above for the 
+        kernel formulas.
+        
+    degree : int
+        Degree parameter for the polynomial
+        kernel. (see formula in the ``gamma``
+        parameter description)
+        
     Attributes
     ----------  
     weights_ : numpy array
@@ -133,10 +171,9 @@ and A. J. Smola. "Correcting sample selection bias by unlabeled data." In NIPS,
     def __init__(self,
                  estimator=None,
                  Xt=None,
-                 yt=None,
+                 kernel="rbf",
                  B=1000,
                  eps=None,
-                 kernel="rbf",
                  max_size=1000,
                  tol=None,
                  max_iter=100,
